const stylistic = require('@stylistic/eslint-plugin');

const isDev = !!process.env?.NODE_ENV?.includes('dev');

const customizedStylisticConfig = stylistic.configs.customize({
  semi: true,
  indent: 2,
  quotes: 'single',
  jsx: true,
  braceStyle: '1tbs',
});

module.exports = {
  env: {
    node: true,
  },
  parserOptions: {
    ecmaVersion: 'latest',
    sourceType: 'module',
  },
  extends: [
    'eslint:recommended',
    'plugin:@stylistic/recommended-extends',
  ],
  rules: {
    ...customizedStylisticConfig.rules,
    ...getJavascriptRules(),
  },
  overrides: [
    {
      env: {
        browser: true,
        es2021: true,
      },
      files: ['src/**/*.{js?(x),ts?(x)}'],
      extends: [
        'plugin:@typescript-eslint/recommended',
        'react-app',
      ],
      plugins: ['react-refresh'],
      parser: '@typescript-eslint/parser',
      parserOptions: {
        ecmaVersion: 'latest',
        sourceType: 'module',
        ecmaFeatures: {
          jsx: true,
        },
      },
      settings: {
        'import/extensions': ['.js', '.jsx', '.ts', '.tsx', '.d.ts'],
        'import/parsers': {
          '@typescript-eslint/parser': ['.ts', '.tsx', '.d.ts'],
        },
        'import/resolver': {
          typescript: {
            alwaysTryTypes: true,
            project: './**/tsconfig.json',
            extensions: ['.js', '.jsx', '.ts', '.tsx', '.d.ts'],
          },
        },
      },
      rules: {
        ...customizedStylisticConfig.rules,
        ...getJavascriptRules(),
        ...getTypescriptRules(),
        ...getA11yRules(),
        ...getImportRules(),
        ...getReactRules(),
        ...getRefreshRules(),
      },
    },
  ],
};

/**
 * Gets accessibility-related rules.
 */
function getA11yRules() {
  return {
    /**
     * Disables the rule that requires click events to have key events.
     * @see https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/main/docs/rules/click-events-have-key-events.md
     */
    'jsx-a11y/click-events-have-key-events': 'off',
  };
}

/**
 * Gets JavaScript-related rules.
 */
function getJavascriptRules() {
  return {
    /**
     * Disables the rule that requires class methods to use `this`.
     * @see https://eslint.org/docs/latest/rules/class-methods-use-this
     */
    'class-methods-use-this': 'off',
  };
}

/**
 * Gets import-related rules.
 */
function getImportRules() {
  return {
    /**
     * Disables the rule that prefers default exports.
     * @see https://github.com/import-js/eslint-plugin-import/blob/main/docs/rules/prefer-default-export.md
     */
    'import/prefer-default-export': 'off',

    /**
     * Sets the rule for file extensions in imports.
     * - Ignores extensions for packages.
     * - Does not require extensions for JS files.
     * - Does not require extensions for TS and TSX files.
     * @see https://github.com/import-js/eslint-plugin-import/blob/main/docs/rules/extensions.md
     */
    'import/extensions': [
      'error',
      'ignorePackages',
      {
        js: 'never',
        ts: 'never',
        tsx: 'never',
      },
    ],
  };
}

/**
 * Gets React-related rules.
 */
function getReactRules() {
  return {
    /**
     * Disables the rule that forbids spreading props in JSX.
     * @see https://github.com/jsx-eslint/eslint-plugin-react/blob/main/docs/rules/jsx-props-no-spreading.md
     */
    'react/jsx-props-no-spreading': 'off',

    /**
     * Disables the rule that requires default props for all props.
     * @see https://github.com/jsx-eslint/eslint-plugin-react/blob/main/docs/rules/require-default-props.md
     */
    'react/require-default-props': 'off',
  };
}

/**
 * Gets React Refresh-related rules.
 */
function getRefreshRules() {
  return {
    /**
     * Warns about exporting anything other than components.
     * @see https://github.com/pmmmwh/react-refresh-typescript#eslint-plugin
     */
    'react-refresh/only-export-components': 'warn',
  };
}

/**
 * Gets TypeScript-related rules.
 */
function getTypescriptRules() {
  return {
    /**
     * Warns about explicit use of `any`.
     * - Allows `any` for rest parameters.
     * - Does not automatically convert `any` to `unknown`.
     * @see https://github.com/typescript-eslint/typescript-eslint/blob/main/packages/eslint-plugin/docs/rules/no-explicit-any.md
     */
    '@typescript-eslint/no-explicit-any': ['warn', {
      ignoreRestArgs: true,
      fixToUnknown: false,
    }],

    /**
     * Warns about array type declaration.
     * - Uses simplified form for default arrays.
     * - Uses simplified form for read-only arrays.
     * @see https://github.com/typescript-eslint/typescript-eslint/blob/main/packages/eslint-plugin/docs/rules/array-type.md
     */
    '@typescript-eslint/array-type': ['warn', {
      default: 'array-simple',
      readonly: 'array-simple',
    }],

    /**
     * Disables the rule that forbids shadowing.
     * @see https://eslint.org/docs/latest/rules/no-shadow
     */
    'no-shadow': 'off',

    /**
     * Enables TypeScript's no-shadow rule.
     * @see https://github.com/typescript-eslint/typescript-eslint/blob/main/packages/eslint-plugin/docs/rules/no-shadow.md
     */
    '@typescript-eslint/no-shadow': 'error',

    /**
     * Disables the rule that forbids unused variables.
     * @see https://eslint.org/docs/latest/rules/no-unused-vars
     */
    'no-unused-vars': 'off',

    /**
     * Enables TypeScript's no-unused-vars rule.
     * - Warns about all unused variables.
     * - Ignores variables starting with `_`.
     * - Ignores rest parameters.
     * @see https://github.com/typescript-eslint/typescript-eslint/blob/main/packages/eslint-plugin/docs/rules/no-unused-vars.md
     */
    '@typescript-eslint/no-unused-vars': isDev
      ? 'off'
      : [
          'error',
          {
            args: 'all',
            argsIgnorePattern: '^_',
            caughtErrors: 'all',
            caughtErrorsIgnorePattern: '^_',
            destructuredArrayIgnorePattern: '^_',
            varsIgnorePattern: '^_',
            ignoreRestSiblings: true,
          },
        ],

    /**
     * Disables the rule that forbids using variables before they are defined.
     * @see https://eslint.org/docs/latest/rules/no-use-before-define
     */
    'no-use-before-define': 'off',

    /**
     * Enables TypeScript's no-use-before-define rule.
     * - Does not warn about using functions before they are defined.
     * - Does not warn about using classes before they are defined.
     * - Warns about using variables before they are defined.
     * - Allows named exports to be used before they are defined.
     * @see https://github.com/typescript-eslint/typescript-eslint/blob/main/packages/eslint-plugin/docs/rules/no-use-before-define.md
     */
    '@typescript-eslint/no-use-before-define': ['error', {
      functions: false,
      classes: false,
      variables: true,
      allowNamedExports: true,
    }],
  };
}
